@@ -5,14 +5,10 @@ import time
 from copy import copy
 import os
 import argparse
-import inspect
 from collections import OrderedDict
 from timeit import default_timer as timer
-
-try:
-    from inspect import getfullargspec
-except ImportError:
-    from inspect import getargspec as getfullargspec
+from typing import Tuple
+from inspect import getfullargspec
 
 import numpy as np
 from numpy import pi, radians, sin, cos, sqrt, clip
@@ -29,6 +25,8 @@ try:
     USE_NUMBA = SAS_NUMBA > 0
     USE_CUDA = SAS_NUMBA > 1
 except ImportError:
+    # Identity decorator @njit or @njit(...)
+    njit = lambda f, *args, **kw: f if callable(f) else (lambda k: k)
     USE_NUMBA = USE_CUDA = False
 
 # Definition of rotation matrices comes from wikipedia:
@@ -39,7 +37,7 @@ def Rx(angle):
     R = [[1, 0, 0],
          [0, +cos(a), -sin(a)],
          [0, +sin(a), +cos(a)]]
-    return np.matrix(R)
+    return np.array(R)
 
 def Ry(angle):
     """Construct a matrix to rotate points about *y* by *angle* degrees."""
@@ -47,7 +45,7 @@ def Ry(angle):
     R = [[+cos(a), 0, +sin(a)],
          [0, 1, 0],
          [-sin(a), 0, +cos(a)]]
-    return np.matrix(R)
+    return np.array(R)
 
 def Rz(angle):
     """Construct a matrix to rotate points about *z* by *angle* degrees."""
@@ -55,7 +53,7 @@ def Rz(angle):
     R = [[+cos(a), -sin(a), 0],
          [+sin(a), +cos(a), 0],
          [0, 0, 1]]
-    return np.matrix(R)
+    return np.array(R)
 
 def pol2rec(r, theta, phi):
     """
@@ -63,40 +61,51 @@ def pol2rec(r, theta, phi):
     """
     theta, phi = radians(theta), radians(phi)
     x = +r * sin(theta) * cos(phi)
-    y = +r * sin(theta)*sin(phi) 
-    z = +r * cos(theta) 
+    y = +r * sin(theta) * sin(phi)
+    z = +r * cos(theta)
     return x, y, z
 
+def jitter(theta, phi, psi):
+    r"""
+    Return the jitter transform to rotate a set of points.
+    View is in degrees using nautical angles with roll $\psi$ around $c$,
+    pitch $\theta$ around $b$ and yaw $\phi$ around $a$.
+
+    **unused**
+    """
+    return Rx(phi) @ Ry(theta) @ Rz(psi)
+
 def rotation(theta, phi, psi):
+    r"""
+    Return a rotation matrix to apply to a set of points.
+    View is in degrees using a $z$-$y$-$z$ rotation sequence of Euler angles
+    $\phi$-$\theta$-$\psi$.  The $c$-axis of the shape starts along $z$ and
+    the $b$-axis starts along $y$.
     """
-    Apply the jitter transform to a set of points.
-    Points are stored in a 3 x n numpy matrix, not a numpy array or tuple.
-    """
-    return Rx(phi)*Ry(theta)*Rz(psi)
-
-def apply_view(points, view):
-    """
-    Apply the view transform (theta, phi, psi) to a set of points.
-    Points are stored in a 3 x n numpy array.
-    View angles are in degrees.
-    """
-    theta, phi, psi = view
-    return np.asarray((Rz(phi)*Ry(theta)*Rz(psi))*np.matrix(points.T)).T
-
+    return Rz(phi) @ Ry(theta) @ Rz(psi)
 
 def invert_view(qx, qy, view):
-    """
-    Return (qa, qb, qc) for the (theta, phi, psi) view angle at detector
-    pixel (qx, qy).
-    View angles are in degrees.
+    r"""
+    Return $(q_a, q_b, q_c)$ for the $(\theta, \phi, \psi)$ view angle at
+    detector pixel corresponding to $(q_x, q_y)$.  View is in degrees using
+    a $z$-$y$-$z$ sequence of Euler angles $\phi$-$\theta$-$\psi$.
     """
     theta, phi, psi = view
+    Rinv = Rz(-psi) @ Ry(-theta) @ Rz(-phi)
     q = np.vstack((qx.flatten(), qy.flatten(), 0*qx.flatten()))
-    return np.asarray((Rz(-psi)*Ry(-theta)*Rz(-phi))*np.matrix(q))
+    return Rinv @ q
 
+def apply_view(points, view):
+    r"""
+    Return $(p_x, p_y, p_x)$ rotated by the $(\theta, \phi, \psi)$ view angle.
+    View is in degrees using a $z$-$y$-$z$ sequence of Euler angles
+    $\phi$-$\theta$-$\psi$.
+    """
+    R = rotation(*view)
+    return points @ R.T
 
 class Shape:
-    rotation = np.matrix([[1., 0, 0], [0, 1, 0], [0, 0, 1]])
+    rotation = np.eye(3)
     center = np.array([0., 0., 0.])[:, None]
     r_max = None
     is_magnetic = False
@@ -106,15 +115,19 @@ class Shape:
         raise NotImplementedError()
 
     def sample(self, density):
-        # type: (float) -> np.ndarray[N], np.ndarray[N, 3]
+        # type: (float) -> Tuple[np.ndarray, np.ndarray]
+        """
+        Returns arrays (rho[N], points[N, 3]).
+        """
         raise NotImplementedError()
 
     def dims(self):
-        # type: () -> float, float, float
+        # type: () -> Tuple[float, float, float]
         raise NotImplementedError()
 
     def rotate(self, theta, phi, psi):
-        self.rotation = rotation(theta, phi, psi) * self.rotation
+        """See :func:`rotation` for details on the rotation matrix."""
+        self.rotation = rotation(theta, phi, psi) @ self.rotation
         return self
 
     def shift(self, x, y, z):
@@ -122,7 +135,7 @@ class Shape:
         return self
 
     def _adjust(self, points):
-        points = np.asarray(self.rotation * np.matrix(points.T)) + self.center
+        points = self.rotation @ points.T + self.center
         return points.T
 
     def r_bins(self, q, over_sampling=1, r_step=None):
@@ -251,7 +264,7 @@ class EllipticalBicelle(Shape):
         radius = points[:, 0]**2 + points[:, 1]**2
         points = points[radius <= 1]
         # set all to core value first
-        values = np.ones_like(points[:, 0])*self.value
+        values = np.full_like(points[:, 0], self.value)
         # then set value to face value if |z| > face/(length/2))
         values[abs(points[:, 2]) > self.length/(self.length + 2*self.thick_face)] = self.value_face
         # finally set value to rim value if outside the core ellipse
@@ -273,9 +286,9 @@ class TruncatedSphere(Shape):
         self.r_max = 2*r if h >= 0 else 2*sqrt(r**2 - h**2)
         self.dims = self.r_max, self.r_max, r+h
         self.volume = pi*(2*r**3/3 + r**2*h - h**3/3)
-        Vp = pi*(2*r**3/3 + r**2*h - h**3/3)
-        Vm = pi*(2*r**3/3 - r**2*h + h**3/3)
-        Vd = Vp + Vm - 4*pi*r**3/3
+        #Vp = pi*(2*r**3/3 + r**2*h - h**3/3)
+        #Vm = pi*(2*r**3/3 - r**2*h + h**3/3)
+        #Vd = Vp + Vm - 4*pi*r**3/3
 
     def sample(self, density):
         num_points = poisson(density*np.prod(self.dims))
@@ -609,50 +622,58 @@ def spin_weights(in_spin, out_spin):
     # needed on the incoming polariser side (assuming that a user), has normalised
     # to the incoming flux with polariser in for SANSPOl and unpolarised beam, respectively.
 
-    weight = [
+    weight = (
         (1.0 - in_spin) * (1.0 - out_spin) / norm, # dd
         (1.0 - in_spin) * out_spin / norm,       # du
         in_spin * (1.0 - out_spin) / norm,       # ud
         in_spin * out_spin / norm,             # uu
-    ]
+    )
     return weight
 
 def orth(A, b_hat): # A = 3 x n, and b_hat unit vector
- return A - np.sum(A*b_hat[:, None], axis=0)[None, :]*b_hat[:, None]    
+    #return A - np.sum(A*b_hat[:, None], axis=0)[None, :]*b_hat[:, None]
+    return A - np.outer(b_hat, b_hat)@A
 
-def magnetic_sld(qx, qy, up_angle, up_phi, rho, rho_m):
+
+def magnetic_sld(qx, qy, up_theta, up_phi, rho, rho_m):
     """
     Compute the complex sld for the magnetic spin states.
     Returns effective rho for spin states [dd, du, ud, uu].
     """
-    # Handle q=0 by setting px = py = 0
-    # Note: this is different from kernel_iq, which I(0,0) to 0
-    q_norm = 1/sqrt(qx**2 + qy**2) if qx != 0. or qy != 0. else 0.
-    cos_spin, sin_spin = cos(radians(up_angle)), sin(radians(up_angle))
+    # For q=0 one would see the demagnetising field of the sample, equivalent
+    # to direction q_hat = [sqrt(1/2), sqrt(1/2), 0] for a disc shaped sample
+    # that is very thin along the beam.
+    # Note: This is different from kernel_iq.c, which sets I(0, 0) to zero.
+    q_norm = sqrt(qx**2 + qy**2)
+    if abs(q_norm) < 1.e-16:
+        q_hat = np.array([1., 1., 0.]) / np.sqrt(2)
+    else:
+        q_hat = np.array([qx, qy, 0]) / q_norm
+    M_perp = orth(rho_m, q_hat)  # M = rho_m
+
+    # perpy_hat and perpz_hat are unit vectors spanning up the plane
+    # perpendicular to polarisation for SF scattering (avoiding
+    # repetitive computation of orthogonal vectors)
+    cos_theta, sin_theta = cos(radians(up_theta)), sin(radians(up_theta))
     cos_phi, sin_phi = cos(radians(up_phi)), sin(radians(up_phi))
-    M = rho_m
-    p_hat = np.array([sin_spin * cos_phi, sin_spin * sin_phi, cos_spin ])
+    p_hat = np.array([sin_theta * cos_phi, sin_theta * sin_phi, cos_theta])
+    perpy_hat = np.array([-sin_phi, cos_phi, 0])
+    perpz_hat = np.array([-cos_theta * cos_phi, -cos_theta * sin_phi, sin_theta])
 
-    
-    q_hat = np.array([qx, qy, 0]) * q_norm
-    M_perp = orth(M,q_hat)
-    M_perpP = orth(M_perp, p_hat)
-    M_perpP_perpQ = orth(M_perpP, q_hat)
+    perpx = p_hat @ M_perp
+    perpy = perpy_hat @ M_perp
+    perpz = perpz_hat @ M_perp
 
-    perpx = np.dot(p_hat, M_perp)
-    perpy = np.sqrt(np.sum(M_perpP_perpQ**2, axis=0))
-    perpz = np.dot(q_hat, M_perpP)
-    
-
-    return [
+    return (
         rho - perpx,   # dd => sld - D M_perpx
         perpy - 1j * perpz, # du => -D (M_perpy + j M_perpz)
         perpy + 1j * perpz, # ud => -D (M_perpy - j M_perpz)
         rho + perpx,   # uu => sld + D M_perpx
-    ]
+    )
 
+# TODO: provide numba and cuda version
 def calc_Iq_magnetic(qx, qy, rho, rho_m, points, volume=1.0, view=(0, 0, 0),
-                     up_frac_i=0.5, up_frac_f=0.5, up_angle=0., up_phi=0.):
+                     up_frac_i=0.5, up_frac_f=0.5, up_theta=0., up_phi=0.):
     """
     *qx*, *qy* correspond to the detector pixels at which to calculate the
     scattering, relative to the beam along the negative z axis.
@@ -666,25 +687,24 @@ def calc_Iq_magnetic(qx, qy, rho, rho_m, points, volume=1.0, view=(0, 0, 0),
     yaw and roll for a beam travelling along the negative z axis.
     *up_frac_i* is the portion of polarizer neutrons which are spin up.
     *up_frac_f* is the portion of analyzer neutrons which are spin up.
-    *up_angle* and *up_phi* are the rotation angle of the spin up direction 
-    in the detector plane and the inclination from the beam direction (z-axis).
+    *up_theta* is the inclination from the beam direction (z-axis).
+    *up_phi* is the rotation in the detector plane.
     *dtype* is the numerical precision of the calculation. [not implemented]
     """
     # TODO: maybe slightly faster to rotate points and rho_m, and drop qc*z
     qx, qy = np.broadcast_arrays(qx, qy)
     qa, qb, qc = invert_view(qx, qy, view)
     rho, volume = np.broadcast_arrays(rho, volume)
-    x, y, z = points.T
     weights = spin_weights(up_frac_i, up_frac_f)
 
     # I(q) = |sum V(r) rho(r) e^(1j q.r)|^2 / sum V(r)
     shape = qx.shape
     Iq = np.zeros(qx.size, 'd')
-    x, y, z, qx, qy = (np.asarray(v, 'd') for v in (x, y, z, qx, qy))
+    x, y, z = points.T
     qx, qy = (v.flatten() for v in (qx, qy))
     for k in range(qx.size):
         ephase = volume*np.exp(1j*(qa[k]*x + qb[k]*y + qc[k]*z))
-        dd, du, ud, uu = magnetic_sld(qx[k], qy[k], up_angle, up_phi, rho, rho_m)
+        dd, du, ud, uu = magnetic_sld(qx[k], qy[k], up_theta, up_phi, rho, rho_m)
         for w, xs in zip(weights, (dd, du, ud, uu)):
             if w == 0.0:
                 continue
@@ -738,11 +758,11 @@ def _calc_Pr_uniform(r, rho, points, volume):
     #print("vol", np.sum(volume))
     return Pr*1e-4
 
-    # Can get an additional 2x by going to C.  Cuda/OpenCL will allow even
+    # Can get an additional 2x by going to C. Cuda/OpenCL will allow even
     # more speedup, though still bounded by the O(n^2) cost.
     """
 void pdfcalc(int n, const double *pts, const double *rho,
-         int nPr, double *Pr, double rstep)
+  int nPr, double *Pr, double rstep)
 {
   int i,j;
   for (i=0; i<n-2; i++) {
@@ -1135,10 +1155,10 @@ def build_csbox(a=10, b=20, c=30, da=1, db=2, dc=3, slda=1, sldb=2, sldc=3, sld_
     return shape, fn, fn_xy
 
 def build_sphere(radius=125, rho=2,
-                 rho_m=0, theta_m=0, phi_m=0, up_i=0, up_f=0, up_angle=0, up_phi=0):
+                 rho_m=0, theta_m=0, phi_m=0, up_i=0, up_f=0, up_theta=0, up_phi=0):
     magnetism = pol2rec(rho_m, theta_m, phi_m) if rho_m != 0.0 else None
     shape = TriaxialEllipsoid(radius, radius, radius, rho, magnetism=magnetism)
-    shape.spin = (up_i, up_f, up_angle, up_phi)
+    shape.spin = (up_i, up_f, up_theta, up_phi)
     fn, fn_xy = wrap_sasmodel(
         'sphere',
         scale=1,
@@ -1151,18 +1171,18 @@ def build_sphere(radius=125, rho=2,
         sld_mphi=phi_m,
         up_frac_i=up_i,
         up_frac_f=up_f,
-        up_angle=up_angle,
+        up_theta=up_theta,
         up_phi=up_phi,
     )
     return shape, fn, fn_xy
 
 def build_ellip(rab=125, rc=50, rho=2,
-                rho_m=0, theta_m=0, phi_m=0, up_i=0, up_f=0, up_angle=0, up_phi=0):
+                rho_m=0, theta_m=0, phi_m=0, up_i=0, up_f=0, up_theta=0, up_phi=0):
     magnetism = pol2rec(rho_m, theta_m, phi_m) if rho_m != 0.0 else None
     shape = TriaxialEllipsoid(rab, rab, rc, rho, magnetism=magnetism)
     # TODO: polarization spec doesn't belong in shape
     # Put spin state info into the shape since we have it available.
-    shape.spin = (up_i, up_f, up_angle, up_phi)
+    shape.spin = (up_i, up_f, up_theta, up_phi)
     fn, fn_xy = wrap_sasmodel(
         'ellipsoid',
         scale=1,
@@ -1176,16 +1196,16 @@ def build_ellip(rab=125, rc=50, rho=2,
         sld_mphi=phi_m,
         up_frac_i=up_i,
         up_frac_f=up_f,
-        up_angle=up_angle,
+        up_theta=up_theta,
         up_phi=up_phi,
     )
     return shape, fn, fn_xy
 
 def build_triell(ra=125, rb=200, rc=50, rho=2,
-                 rho_m=0, theta_m=0, phi_m=0, up_i=0, up_f=0, up_angle=0, up_phi=0):
+                 rho_m=0, theta_m=0, phi_m=0, up_i=0, up_f=0, up_theta=0, up_phi=0):
     magnetism = pol2rec(rho_m, theta_m, phi_m) if rho_m != 0.0 else None
     shape = TriaxialEllipsoid(ra, rb, rc, rho, magnetism=magnetism)
-    shape.spin = (up_i, up_f, up_angle, up_phi)
+    shape.spin = (up_i, up_f, up_theta, up_phi)
     fn, fn_xy = wrap_sasmodel(
         'triaxial_ellipsoid',
         scale=1,
@@ -1200,8 +1220,8 @@ def build_triell(ra=125, rb=200, rc=50, rho=2,
         sld_mphi=phi_m,
         up_frac_i=up_i,
         up_frac_f=up_f,
-        up_angle=up_angle,
-        up_phi=up_phi,        
+        up_theta=up_theta,
+        up_phi=up_phi,
     )
     return shape, fn, fn_xy
 
@@ -1211,6 +1231,10 @@ def build_cylinder(radius=25, length=125, rho=2.):
     fn_xy = lambda qx, qy, view: cylinder_Iqxy(qx, qy, radius, length, view=view)*rho**2
     return shape, fn, fn_xy
 
+def build_truncated_sphere(radius=25, h=0.5, rho=2.):
+    shape = TruncatedSphere(radius, h=radius*h, value=rho)
+    return shape, None, None
+
 def build_ellcyl(ra=25, rb=50, length=125, rho=2.):
     shape = EllipticalCylinder(ra, rb, length, rho)
     fn, fn_xy = wrap_sasmodel(
@@ -1297,6 +1321,7 @@ SHAPE_FUNCTIONS = OrderedDict([
     ("barbell", build_barbell),
     ("capcyl", build_capcyl),
     ("sphere", build_sphere),
+    ("tsphere", build_truncated_sphere),
     ("box", build_box),
     ("csbox", build_csbox),
     ("cscyl", build_cscyl),
@@ -1324,9 +1349,10 @@ def check_shape(title, shape, fn=None, show_points=False,
 
     import pylab
     if show_points:
-         plot_points(rho, points); pylab.figure()
+        plot_points(rho, points)
+        pylab.figure()
     plot_calc(r, Pr, q, Iq, theory=theory, title=title, Iq_avg=Iq_avg)
-    pylab.gcf().canvas.set_window_title(title)
+    pylab.gcf().canvas.manager.set_window_title(title)
     pylab.show()
 
 def check_shape_2d(title, shape, fn=None, view=(0, 0, 0), show_points=False,
@@ -1337,8 +1363,15 @@ def check_shape_2d(title, shape, fn=None, view=(0, 0, 0), show_points=False,
     qx = np.linspace(-qmax, qmax, mesh)
     qy = np.linspace(-qmax, qmax, mesh)
     Qx, Qy = np.meshgrid(qx, qy)
-    sampling_density = samples / shape.volume
     t0 = timer()
+    theory = fn(Qx, Qy, view) if fn is not None else None
+    print("calc theory time", timer() - t0)
+
+    t0 = timer()
+    sampling_density = samples / shape.volume
+    if False: # point orientation test: rotate shape rather than view
+        shape.rotate(*view)
+        view = (0, 0, 0)
     rho, points = shape.sample(sampling_density)
     # The volume of each sample is approximately 1/sampling_density, except
     # that the number of points actually sampled may be slightly more or
@@ -1351,9 +1384,6 @@ def check_shape_2d(title, shape, fn=None, view=(0, 0, 0), show_points=False,
     t0 = timer()
     Iqxy = calc_Iqxy(Qx, Qy, rho, points, volume=volume, view=view)
     print("calc Iqxy time", timer() - t0)
-    t0 = timer()
-    theory = fn(Qx, Qy, view) if fn is not None else None
-    print("calc theory time", timer() - t0)
 
     # Add floor to limit colorbar range.
     Iqxy += 0.001 * Iqxy.max()
@@ -1362,9 +1392,10 @@ def check_shape_2d(title, shape, fn=None, view=(0, 0, 0), show_points=False,
 
     import pylab
     if show_points:
-        plot_points(rho, points); pylab.figure()
+        plot_points(rho, apply_view(points, view))
+        pylab.figure()
     plot_calc_2d(qx, qy, Iqxy, theory=theory, title=title)
-    pylab.gcf().canvas.set_window_title(title)
+    pylab.gcf().canvas.manager.set_window_title(title)
 
     ## Histogram of point density in the z direction.
     #pylab.figure()
@@ -1376,7 +1407,7 @@ def check_shape_2d(title, shape, fn=None, view=(0, 0, 0), show_points=False,
 
 def check_shape_mag(title, shape, fn=None, view=(0, 0, 0), show_points=False,
                     mesh=100, qmax=1.0, samples=5000,
-                    up_frac_i=0, up_frac_f=0, up_angle=0, up_phi=0):
+                    up_frac_i=0, up_frac_f=0, up_theta=0, up_phi=0):
     rho_solvent = 0
     #qx = np.linspace(0.0, qmax, mesh)
     #qy = np.linspace(0.0, qmax, mesh)
@@ -1395,7 +1426,7 @@ def check_shape_mag(title, shape, fn=None, view=(0, 0, 0), show_points=False,
     t0 = timer()
     Iqxy = calc_Iq_magnetic(Qx, Qy, rho, rho_m, points, volume=volume, view=view,
                             up_frac_i=up_frac_i, up_frac_f=up_frac_f,
-                            up_angle=up_angle, up_phi=up_phi)
+                            up_theta=up_theta, up_phi=up_phi)
     print("calc I_mag time", timer() - t0)
     t0 = timer()
     theory = fn(Qx, Qy, view) if fn is not None else None
@@ -1408,9 +1439,10 @@ def check_shape_mag(title, shape, fn=None, view=(0, 0, 0), show_points=False,
 
     import pylab
     if show_points:
-        plot_points(rho, points); pylab.figure()
+        plot_points(rho, points)
+        pylab.figure()
     plot_calc_2d(qx, qy, Iqxy, theory=theory, title=title)
-    pylab.gcf().canvas.set_window_title(title)
+    pylab.gcf().canvas.manager.set_window_title(title)
 
     ## Histogram of point density in the z direction.
     #pylab.figure()
@@ -1473,10 +1505,10 @@ def main():
     title = "%s(%s)" % (opts.shape, " ".join(opts.pars))
     if shape.is_magnetic:
         view = tuple(float(v) for v in opts.view.split(','))
-        up_frac_i, up_frac_f, up_angle, up_phi = shape.spin
+        up_frac_i, up_frac_f, up_theta, up_phi = shape.spin
         check_shape_mag(title, shape, fn_xy, view=view, show_points=opts.plot,
                        mesh=opts.mesh, qmax=opts.qmax, samples=opts.samples,
-                       up_frac_i=up_frac_i, up_frac_f=up_frac_f, up_angle=up_angle, up_phi=up_phi,
+                       up_frac_i=up_frac_i, up_frac_f=up_frac_f, up_theta=up_theta, up_phi=up_phi,
                        )
     elif opts.dim == 1:
         check_shape(title, shape, fn, show_points=opts.plot,
